@@ -1,6 +1,6 @@
 /* The RELP (reliable event logging protocol) core protocol library.
  *
- * Copyright 2008-2019 by Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2008-2020 by Rainer Gerhards and Adiscon GmbH.
  *
  * This file is part of librelp.
  *
@@ -52,7 +52,7 @@
 /* ------------------------------ some internal functions ------------------------------ */
 
 void LIBRELP_ATTR_FORMAT(printf, 4, 5)
-relpEngineCallOnGenericErr(relpEngine_t *pThis, char *eobj, relpRetVal ecode, char *fmt, ...)
+relpEngineCallOnGenericErr(relpEngine_t *const pThis, const char *eobj, relpRetVal ecode, const char *fmt, ...)
 {
 	va_list ap;
 	char emsg[1024];
@@ -62,10 +62,10 @@ relpEngineCallOnGenericErr(relpEngine_t *pThis, char *eobj, relpRetVal ecode, ch
 	emsg[sizeof(emsg)/sizeof(char) - 1] = '\0'; /* just to be on the safe side... */
 	va_end(ap);
 
-	pThis->dbgprint("librelp: generic error: ecode %d, eobj %s,"
+	pThis->dbgprint((char*)"librelp: generic error: ecode %d, eobj %s,"
 		"emsg '%s'\n", ecode, eobj, emsg);
 	if(pThis->onGenericErr != NULL) {
-		pThis->onGenericErr(eobj, emsg, ecode);
+		pThis->onGenericErr((char*)eobj, emsg, ecode);
 	}
 }
 
@@ -93,7 +93,7 @@ _relpEngine_strerror_r(const int errnum, char *buf, const size_t buflen) {
 
 #if defined(HAVE_EPOLL_CREATE1) || defined(HAVE_EPOLL_CREATE)
 static relpRetVal
-addToEpollSet(relpEngine_t *pThis, epolld_type_t typ, void *ptr, int sock, epolld_t **pepd)
+addToEpollSet(relpEngine_t *const pThis, epolld_type_t typ, void *ptr, int sock, epolld_t **pepd)
 {
 	epolld_t *epd = NULL;
 	ENTER_RELPFUNC;
@@ -105,7 +105,7 @@ addToEpollSet(relpEngine_t *pThis, epolld_type_t typ, void *ptr, int sock, epoll
 	epd->ev.events = EPOLLIN;
 	epd->ev.data.ptr = (void*) epd;
 
-	pThis->dbgprint("librelp: add socket %d to epoll set (ptr %p)\n", sock, ptr);
+	pThis->dbgprint((char*)"librelp: add socket %d to epoll set (ptr %p)\n", sock, ptr);
 	if(epoll_ctl(pThis->efd, EPOLL_CTL_ADD, sock, &epd->ev) != 0) {
 		char errStr[1024];
 		int eno = errno;
@@ -127,10 +127,10 @@ finalize_it:
  * in any case...
  */
 static void
-delFromEpollSet(relpEngine_t *pThis, epolld_t *epd)
+delFromEpollSet(relpEngine_t *const pThis, epolld_t *epd)
 {
 	int r;
-	pThis->dbgprint("librelp: delete sock %d from epoll set\n", epd->sock);
+	pThis->dbgprint((char*)"librelp: delete sock %d from epoll set\n", epd->sock);
 	if((r = epoll_ctl(pThis->efd, EPOLL_CTL_DEL, epd->sock, &epd->ev)) != 0) {
 		char errStr[1024];
 		int eno = errno;
@@ -142,7 +142,7 @@ delFromEpollSet(relpEngine_t *pThis, epolld_t *epd)
 }
 
 static relpRetVal
-addSessToEpoll(relpEngine_t *pThis, relpEngSessLst_t *pSessLstEntry)
+addSessToEpoll(relpEngine_t *const pThis, relpEngSessLst_t *pSessLstEntry)
 {
 	addToEpollSet(pThis, epolld_sess, pSessLstEntry,
 		relpSessGetSock(pSessLstEntry->pSess), &pSessLstEntry->epevt);
@@ -151,7 +151,7 @@ addSessToEpoll(relpEngine_t *pThis, relpEngSessLst_t *pSessLstEntry)
 }
 
 static void
-delSessFromEpoll(relpEngine_t *pThis, relpEngSessLst_t *pSessEtry)
+delSessFromEpoll(relpEngine_t *const pThis, relpEngSessLst_t *pSessEtry)
 {
 	delFromEpollSet(pThis, pSessEtry->epevt);
 }
@@ -162,7 +162,7 @@ delSessFromEpoll(relpEngine_t *pThis, relpEngSessLst_t *pSessEtry)
  * rgerhards, 2008-03-17
  */
 static relpRetVal
-relpEngineAddToSrvList(relpEngine_t *pThis, relpSrv_t *pSrv)
+relpEngineAddToSrvList(relpEngine_t *const pThis, relpSrv_t *pSrv)
 {
 	relpEngSrvLst_t *pSrvLstEntry;
 
@@ -190,7 +190,7 @@ finalize_it:
  * rgerhards, 2008-03-17
  */
 static relpRetVal
-relpEngineAddToSess(relpEngine_t *pThis, relpSess_t *pSess)
+relpEngineAddToSess(relpEngine_t *const pThis, relpSess_t *pSess)
 {
 	relpEngSessLst_t *pSessLstEntry;
 
@@ -220,7 +220,7 @@ finalize_it:
  * rgerhards, 2008-03-17
  */
 static relpRetVal
-relpEngineDelSess(relpEngine_t *pThis, relpEngSessLst_t *pSessLstEntry)
+relpEngineDelSess(relpEngine_t *const pThis, relpEngSessLst_t *pSessLstEntry)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Engine);
@@ -247,7 +247,7 @@ relpEngineDelSess(relpEngine_t *pThis, relpEngSessLst_t *pSessLstEntry)
  * RELP function. The relp engine must only destructed after all RELP
  * operations have been finished.
  */
-relpRetVal
+relpRetVal PART_OF_API
 relpEngineConstruct(relpEngine_t **ppThis)
 {
 	relpEngine_t *pThis;
@@ -281,7 +281,7 @@ finalize_it:
  * Terminates librelp operations, no calls are permitted after engine
  * destruction.
  */
-relpRetVal
+relpRetVal PART_OF_API
 relpEngineDestruct(relpEngine_t **ppThis)
 {
 	relpEngine_t *pThis;
@@ -307,6 +307,9 @@ relpEngineDestruct(relpEngine_t **ppThis)
 		free(pSrvL);
 	}
 
+	/* destruct TLS structures */
+	relpTcpExitTLS();
+
 	pthread_mutex_destroy(&pThis->mutSrvLst);
 	pthread_mutex_destroy(&pThis->mutSessLst);
 	/* done with de-init work, now free engine object itself */
@@ -322,8 +325,8 @@ static void dbgprintDummy(char LIBRELP_ATTR_UNUSED *fmt, ...) {}
  * function that already has been set, provide a NULL function pointer.
  * rgerhards, 2008-03-17
  */
-relpRetVal
-relpEngineSetDbgprint(relpEngine_t *pThis, void (*dbgprint)(char *fmt, ...) LIBRELP_ATTR_FORMAT(printf, 1, 2))
+relpRetVal PART_OF_API
+relpEngineSetDbgprint(relpEngine_t *const pThis, void (*dbgprint)(char *fmt, ...) LIBRELP_ATTR_FORMAT(printf, 1, 2))
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Engine);
@@ -333,7 +336,7 @@ relpEngineSetDbgprint(relpEngine_t *pThis, void (*dbgprint)(char *fmt, ...) LIBR
 }
 
 
-relpRetVal
+relpRetVal PART_OF_API
 relpEngineSetTLSLib(relpEngine_t *const pThis, NOTLS_UNUSED const int tls_lib)
 {
 	ENTER_RELPFUNC;
@@ -343,7 +346,7 @@ relpEngineSetTLSLib(relpEngine_t *const pThis, NOTLS_UNUSED const int tls_lib)
 		ABORT_FINALIZE(RELP_RET_PARAM_ERROR);
 	}
 
-	pThis->dbgprint("relpEngineSetTLSLib, req lib is %d\n", tls_lib);
+	pThis->dbgprint((char*)"relpEngineSetTLSLib, req lib is %d\n", tls_lib);
 	#if defined(WITH_TLS)
 		if(tls_lib == RELP_USE_GNUTLS) {
 			#if defined(ENABLE_TLS)
@@ -361,17 +364,18 @@ relpEngineSetTLSLib(relpEngine_t *const pThis, NOTLS_UNUSED const int tls_lib)
 			ABORT_FINALIZE(RELP_RET_PARAM_ERROR);
 		}
 	#else /* defined(WITH_TLS) */
-		pThis->dbgprint("relpEngineSetTLSLib, error, librelp not with TLS support\n");
+		pThis->dbgprint((char*)"relpEngineSetTLSLib, error, librelp not with TLS support\n");
 		ABORT_FINALIZE(RELP_RET_NOT_SUPPORTED);
 	#endif
 
 finalize_it:
-	pThis->dbgprint("relpEngineSetTLSLib, lib now %d, ret %d\n", pThis->tls_lib, iRet);
+	if (pThis)
+		pThis->dbgprint((char*)"relpEngineSetTLSLib, lib now %d, ret %d\n", pThis->tls_lib, iRet);
 	LEAVE_RELPFUNC;
 }
 
 
-relpRetVal
+relpRetVal PART_OF_API
 relpEngineSetTLSLibByName(relpEngine_t *const pThis, const char *const name)
 {
 	ENTER_RELPFUNC;
@@ -382,9 +386,9 @@ relpEngineSetTLSLibByName(relpEngine_t *const pThis, const char *const name)
 	}
 
 	if(!strcasecmp(name, "gnutls")) {
-		relpEngineSetTLSLib(pThis, RELP_USE_GNUTLS);
+		CHKRet(relpEngineSetTLSLib(pThis, RELP_USE_GNUTLS));
 	}else if(!strcasecmp(name, "openssl")) {
-		relpEngineSetTLSLib(pThis, RELP_USE_OPENSSL);
+		CHKRet(relpEngineSetTLSLib(pThis, RELP_USE_OPENSSL));
 	} else {
 		relpEngineCallOnGenericErr(pThis, "librelp", RELP_RET_PARAM_ERROR,
 				"invalid tls lib '%s' requested; this version of "
@@ -405,8 +409,8 @@ finalize_it:
  * * relgEngineListnerConstructFinalize()
  * rgerhards, 2013-05-14
  */
-relpRetVal
-relpEngineListnerConstruct(relpEngine_t *pThis, relpSrv_t **ppSrv)
+relpRetVal PART_OF_API
+relpEngineListnerConstruct(relpEngine_t *const pThis, relpSrv_t **ppSrv)
 {
 	relpSrv_t *pSrv;
 	ENTER_RELPFUNC;
@@ -418,8 +422,8 @@ relpEngineListnerConstruct(relpEngine_t *pThis, relpSrv_t **ppSrv)
 finalize_it:
 	LEAVE_RELPFUNC;
 }
-relpRetVal
-relpEngineListnerConstructFinalize(relpEngine_t *pThis, relpSrv_t *pSrv)
+relpRetVal PART_OF_API
+relpEngineListnerConstructFinalize(relpEngine_t *const pThis, relpSrv_t *pSrv)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Engine);
@@ -446,8 +450,8 @@ static relpRetVal relpSrvSyslogRcvDummy2(void LIBRELP_ATTR_UNUSED *pUsr,
 /* set the syslog receive callback. If NULL is provided, it is set to the
  * not implemented dummy.
  */
-relpRetVal
-relpEngineSetSyslogRcv2(relpEngine_t *pThis, relpRetVal (*pCB)(void *, unsigned char*,
+relpRetVal PART_OF_API
+relpEngineSetSyslogRcv2(relpEngine_t *const pThis, relpRetVal (*pCB)(void *, unsigned char*,
 	unsigned char*, unsigned char*, size_t))
 {
 	ENTER_RELPFUNC;
@@ -479,8 +483,8 @@ relpEngineSetSyslogRcv2(relpEngine_t *pThis, relpRetVal (*pCB)(void *, unsigned
  * errmsg   - error message as far as librelp is concerned
  * errcode  - contains librelp error status that lead to the failed auth.
  */
-relpRetVal
-relpEngineSetOnAuthErr(relpEngine_t *pThis, void (*pCB)(void*pUsr, char *authinfo, char*errmsg, relpRetVal errcode) )
+relpRetVal PART_OF_API
+relpEngineSetOnAuthErr(relpEngine_t *const pThis, void (*pCB)(void*pUsr, char *authinfo, char*errmsg, relpRetVal errcode) )
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Engine);
@@ -503,8 +507,8 @@ relpEngineSetOnAuthErr(relpEngine_t *pThis, void (*pCB)(void*pUsr, char *authinf
  * errmsg   - error message as far as librelp is concerned
  * errcode  - contains librelp error status
  */
-relpRetVal
-relpEngineSetOnErr(relpEngine_t *pThis, void (*pCB)(void*pUsr, char *objinfo, char*errmsg, relpRetVal errcode) )
+relpRetVal PART_OF_API
+relpEngineSetOnErr(relpEngine_t *const pThis, void (*pCB)(void*pUsr, char *objinfo, char*errmsg, relpRetVal errcode) )
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Engine);
@@ -523,8 +527,8 @@ relpEngineSetOnErr(relpEngine_t *pThis, void (*pCB)(void*pUsr, char *objinfo, ch
  * errmsg   - error message as far as librelp is concerned
  * errcode  - contains librelp error status
  */
-relpRetVal
-relpEngineSetOnGenericErr(relpEngine_t *pThis, void (*pCB)(char *objinfo, char*errmsg, relpRetVal errcode) )
+relpRetVal PART_OF_API
+relpEngineSetOnGenericErr(relpEngine_t *const pThis, void (*pCB)(char *objinfo, char*errmsg, relpRetVal errcode) )
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Engine);
@@ -535,8 +539,8 @@ relpEngineSetOnGenericErr(relpEngine_t *pThis, void (*pCB)(char *objinfo, char*e
 /* Deprecated, use relpEngineListnerConstruct() family of functions.
  * See there for further information.
  */
-relpRetVal
-relpEngineAddListner2(relpEngine_t *pThis, unsigned char *pLstnPort, void *pUsr)
+relpRetVal PART_OF_API
+relpEngineAddListner2(relpEngine_t *const pThis, unsigned char *pLstnPort, void *pUsr)
 {
 	relpSrv_t *pSrv = NULL;
 	ENTER_RELPFUNC;
@@ -563,8 +567,8 @@ static relpRetVal relpSrvSyslogRcvDummy(unsigned char LIBRELP_ATTR_UNUSED *pHost
 /* set the syslog receive callback. If NULL is provided, it is set to the
  * not implemented dummy.
  */
-relpRetVal
-relpEngineSetSyslogRcv(relpEngine_t *pThis, relpRetVal (*pCB)(unsigned char*, unsigned char*, unsigned char*, size_t))
+relpRetVal PART_OF_API
+relpEngineSetSyslogRcv(relpEngine_t *const pThis, relpRetVal (*pCB)(unsigned char*, unsigned char*, unsigned char*, size_t))
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Engine);
@@ -577,8 +581,8 @@ relpEngineSetSyslogRcv(relpEngine_t *pThis, relpRetVal (*pCB)(unsigned char*, un
 /* Deprecated, use relpEngineListnerConstruct() family of functions.
  * See there for further information.
  */
-relpRetVal
-relpEngineAddListner(relpEngine_t *pThis, unsigned char *pLstnPort)
+relpRetVal PART_OF_API
+relpEngineAddListner(relpEngine_t *const pThis, unsigned char *pLstnPort)
 {
 	relpSrv_t *pSrv = NULL;
 	ENTER_RELPFUNC;
@@ -601,7 +605,8 @@ finalize_it:
  * signal is sent, it may take rather long to stop the server (until another
  * machine sends data).
  */
-relpRetVal relpEngineSetStop(relpEngine_t *pThis)
+relpRetVal PART_OF_API
+relpEngineSetStop(relpEngine_t *const pThis)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Engine);
@@ -612,7 +617,8 @@ relpRetVal relpEngineSetStop(relpEngine_t *pThis)
 
 /* set the socket family to use
  */
-relpRetVal relpEngineSetFamily(relpEngine_t *pThis, int ai_family)
+relpRetVal PART_OF_API
+relpEngineSetFamily(relpEngine_t *const pThis, int ai_family)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Engine);
@@ -623,8 +629,8 @@ relpRetVal relpEngineSetFamily(relpEngine_t *pThis, int ai_family)
 /* helper for relpEngineRun; receives data when it is time to
  * do so. Includes all housekeeping, like closing the session.
  */
-static inline relpRetVal
-doRecv(relpEngine_t *pThis, relpEngSessLst_t *pSessEtry, int sock)
+static relpRetVal
+doRecv(relpEngine_t *const pThis, relpEngSessLst_t *pSessEtry, int sock)
 {
 	relpRetVal localRet;
 	localRet = relpSessRcvData(pSessEtry->pSess); /* errors are handled there */
@@ -633,7 +639,7 @@ doRecv(relpEngine_t *pThis, relpEngSessLst_t *pSessEtry, int sock)
 	 * session, which may eventually be followed by a new connect.
 	 */
 	if(localRet != RELP_RET_OK) {
-		pThis->dbgprint("relp session %d iRet %d, tearing it down\n",
+		pThis->dbgprint((char*)"relp session %d iRet %d, tearing it down\n",
 				sock, localRet);
 		relpEngineDelSess(pThis, pSessEtry);
 	}
@@ -642,8 +648,8 @@ doRecv(relpEngine_t *pThis, relpEngSessLst_t *pSessEtry, int sock)
 /* helper for relpEngineRun; sends session data when it is time
  * to send. Includes all housekeeping, like closing the session.
  */
-static inline void
-doSend(relpEngine_t *pThis, relpEngSessLst_t *pSessEtry, int sock)
+static void
+doSend(relpEngine_t *const pThis, relpEngSessLst_t *pSessEtry, int sock)
 {
 	relpRetVal localRet;
 	localRet = relpSessSndData(pSessEtry->pSess); /* errors are handled there */
@@ -652,19 +658,19 @@ doSend(relpEngine_t *pThis, relpEngSessLst_t *pSessEtry, int sock)
 	 * session, which may eventually be followed by a new connect.
 	 */
 	if(localRet != RELP_RET_OK) {
-		pThis->dbgprint("relp session %d iRet %d during send, tearing it down\n",
+		pThis->dbgprint((char*)"relp session %d iRet %d during send, tearing it down\n",
 				sock, localRet);
 		relpEngineDelSess(pThis, pSessEtry);
 	}
 }
 
 static void
-handleConnectionRequest(relpEngine_t *pThis, relpSrv_t *pSrv, int sock)
+handleConnectionRequest(relpEngine_t *const pThis, relpSrv_t *pSrv, int sock)
 {
 	relpRetVal localRet;
 	relpSess_t *pNewSess;
 
-	pThis->dbgprint("new connect on RELP socket #%d\n", sock);
+	pThis->dbgprint((char*)"new connect on RELP socket #%d\n", sock);
 	localRet = relpSessAcceptAndConstruct(&pNewSess, pSrv, sock);
 	if(localRet == RELP_RET_OK) {
 		relpEngineAddToSess(pThis, pNewSess);
@@ -675,15 +681,15 @@ handleConnectionRequest(relpEngine_t *pThis, relpSrv_t *pSrv, int sock)
 #if defined(HAVE_EPOLL_CREATE1) || defined(HAVE_EPOLL_CREATE)
 
 static relpRetVal
-epoll_set_events(relpEngine_t *pThis, relpEngSessLst_t *pSessEtry, int sock, uint32_t events)
+epoll_set_events(relpEngine_t *const pThis, relpEngSessLst_t *pSessEtry, int sock, uint32_t events)
 {
 	ENTER_RELPFUNC;
 	/* TODO: remove the status dbgprint's once we have some practice drill 2013-07-05 */
-	pThis->dbgprint("librelp: epoll_set_events sock %d, target bits %2.2x, current %2.2x\n",
+	pThis->dbgprint((char*)"librelp: epoll_set_events sock %d, target bits %2.2x, current %2.2x\n",
 		sock, events, pSessEtry->epevt->ev.events);
 	if(pSessEtry->epevt->ev.events != events) {
 		pSessEtry->epevt->ev.events = events;
-		pThis->dbgprint("librelp: epoll_set_events sock %d, setting new bits\n", sock);
+		pThis->dbgprint((char*)"librelp: epoll_set_events sock %d, setting new bits\n", sock);
 		if(epoll_ctl(pThis->efd, EPOLL_CTL_MOD, sock, &pSessEtry->epevt->ev) != 0) {
 			char errStr[1024];
 			int eno = errno;
@@ -697,7 +703,7 @@ finalize_it:
 	LEAVE_RELPFUNC;
 }
 
-static inline relpRetVal
+static relpRetVal
 engineEventLoopInit(relpEngine_t LIBRELP_ATTR_UNUSED *pThis)
 {
 #	define NUM_EPOLL_EVENTS 10
@@ -715,7 +721,7 @@ engineEventLoopInit(relpEngine_t LIBRELP_ATTR_UNUSED *pThis)
 	}
 
 	if(pThis->efd < 0) {
-		pThis->dbgprint("epoll_create1() could not create fd\n");
+		pThis->dbgprint((char*)"epoll_create1() could not create fd\n");
 		ABORT_FINALIZE(RELP_RET_IO_ERR);
 	}
 
@@ -734,7 +740,7 @@ engineEventLoopInit(relpEngine_t LIBRELP_ATTR_UNUSED *pThis)
 finalize_it:
 	LEAVE_RELPFUNC;
 }
-static inline relpRetVal
+static relpRetVal
 engineEventLoopExit(relpEngine_t LIBRELP_ATTR_UNUSED *pThis)
 {
 	relpEngSrvLst_t *pSrvEtry;
@@ -754,7 +760,7 @@ engineEventLoopExit(relpEngine_t LIBRELP_ATTR_UNUSED *pThis)
 }
 
 static relpRetVal
-handleSessIO(relpEngine_t *pThis, epolld_t *epd)
+handleSessIO(relpEngine_t *const pThis, epolld_t *epd)
 {
 	relpEngSessLst_t *pSessEtry;
 
@@ -769,12 +775,12 @@ handleSessIO(relpEngine_t *pThis, epolld_t *epd)
 #			if defined(ENABLE_TLS) || defined(ENABLE_TLS_OPENSSL)
 				const relpRetVal localRet = relpTcpRtryHandshake(pTcp);
 				if(localRet != RELP_RET_OK) {
-					pThis->dbgprint("relp session %d handshake iRet %d, tearing it down\n",
+					pThis->dbgprint((char*)"relp session %d handshake iRet %d, tearing it down\n",
 							epd->sock, localRet);
 					relpEngineDelSess(pThis, pSessEtry);
 				}
 #			else
-					pThis->dbgprint("librelp error: handshake retry requested in "
+					pThis->dbgprint((char*)"librelp error: handshake retry requested in "
 							"non-TLS mode");
 
 #			endif /* #ifdef ENABLE_TLS | ENABLE_TLS_OPENSSL*/
@@ -790,7 +796,7 @@ handleSessIO(relpEngine_t *pThis, epolld_t *epd)
 }
 
 static relpRetVal
-engineEventLoopRun(relpEngine_t *pThis)
+engineEventLoopRun(relpEngine_t *const pThis)
 {
 	relpEngSessLst_t *pSessEtry;
 	int i;
@@ -811,7 +817,7 @@ engineEventLoopRun(relpEngine_t *pThis)
 			sock = relpSessGetSock(pSessEtry->pSess);
 #			if defined(ENABLE_TLS) || defined(ENABLE_TLS_OPENSSL)
 			if(relpSessTcpRequiresRtry(pSessEtry->pSess)) {
-				pThis->dbgprint("librelp: retry op requested for sock %d\n", sock);
+				pThis->dbgprint((char*)"librelp: retry op requested for sock %d\n", sock);
 				if(relpTcpGetRtryDirection(pSessEtry->pSess->pTcp) == 0) {
 					epoll_set_events(pThis, pSessEtry, sock, EPOLLIN);
 				} else {
@@ -831,9 +837,9 @@ engineEventLoopRun(relpEngine_t *pThis)
 
 		/* wait for io to become ready */
 		if(relpEngineShouldStop(pThis)) break;
-		pThis->dbgprint("librelp: doing epoll_wait\n");
+		pThis->dbgprint((char*)"librelp: doing epoll_wait\n");
 		const int nEvents = epoll_wait(pThis->efd, events, sizeof(events)/sizeof(struct epoll_event), -1);
-		pThis->dbgprint("librelp: done epoll_wait, nEvents:%d\n", nEvents);
+		pThis->dbgprint((char*)"librelp: done epoll_wait, nEvents:%d\n", nEvents);
 		if(relpEngineShouldStop(pThis)) break;
 
 		for(i = 0 ; i < nEvents ; ++i) {
@@ -860,7 +866,7 @@ engineEventLoopRun(relpEngine_t *pThis)
 static inline relpRetVal engineEventLoopInit(relpEngine_t LIBRELP_ATTR_UNUSED *pThis) { return RELP_RET_OK; }
 static inline relpRetVal engineEventLoopExit(relpEngine_t LIBRELP_ATTR_UNUSED *pThis) { return RELP_RET_OK; }
 static relpRetVal
-engineEventLoopRun(relpEngine_t *pThis)
+engineEventLoopRun(relpEngine_t *const pThis)
 {
 	relpEngSrvLst_t *pSrvEtry;
 	relpEngSessLst_t *pSessEtry;
@@ -896,7 +902,7 @@ engineEventLoopRun(relpEngine_t *pThis)
 			sock = relpSessGetSock(pSessEtry->pSess);
 #			if defined(ENABLE_TLS) || defined(ENABLE_TLS_OPENSSL)
 			if(relpSessTcpRequiresRtry(pSessEtry->pSess)) {
-				pThis->dbgprint("librelp: retry op requested for sock %d\n", sock);
+				pThis->dbgprint((char*)"librelp: retry op requested for sock %d\n", sock);
 				if(relpTcpGetRtryDirection(pSessEtry->pSess->pTcp) == 0) {
 					FD_SET(sock, &readfds);
 				} else {
@@ -916,24 +922,24 @@ engineEventLoopRun(relpEngine_t *pThis)
 
 		/* done adding all sockets */
 		if(pThis->dbgprint != dbgprintDummy) {
-			pThis->dbgprint("librelp: calling select, active file descriptors (max %d): ", maxfds);
+			pThis->dbgprint((char*)"librelp: calling select, active file descriptors (max %d): ", maxfds);
 			for(nfds = 0; nfds <= maxfds; ++nfds)
 				if(FD_ISSET(nfds, &readfds))
-					pThis->dbgprint("%d ", nfds);
-			pThis->dbgprint("\n");
+					pThis->dbgprint((char*)"%d ", nfds);
+			pThis->dbgprint((char*)"\n");
 		}
 
 		/* wait for io to become ready */
 		if(relpEngineShouldStop(pThis)) break;
 		nfds = select(maxfds+1, (fd_set *) &readfds, &writefds, NULL, NULL);
-		pThis->dbgprint("relp select returns, nfds %d\n", nfds);
+		pThis->dbgprint((char*)"relp select returns, nfds %d\n", nfds);
 		if(relpEngineShouldStop(pThis)) break;
 
 		if(nfds == -1) {
 			if(errno == EINTR) {
-				pThis->dbgprint("relp select interrupted\n");
+				pThis->dbgprint((char*)"relp select interrupted\n");
 			} else {
-				pThis->dbgprint("relp select returned error %d\n", errno);
+				pThis->dbgprint((char*)"relp select returned error %d\n", errno);
 			}
 			continue;
 		}
@@ -968,13 +974,13 @@ engineEventLoopRun(relpEngine_t *pThis)
 #						if defined(ENABLE_TLS) || defined(ENABLE_TLS_OPENSSL)
 							localRet = relpTcpRtryHandshake(pSessEtry->pSess->pTcp);
 							if(localRet != RELP_RET_OK) {
-								pThis->dbgprint("relp session %d handshake "
+								pThis->dbgprint((char*)"relp session %d handshake "
 										"iRet %d, tearing it down\n",
 										sock, localRet);
 								relpEngineDelSess(pThis, pSessEtry);
 							}
 #						else
-							pThis->dbgprint("librelp error: handshake retry "
+							pThis->dbgprint((char*)"librelp error: handshake retry "
 									"requested in non-TLS mode");
 
 #						endif /* #ifdef ENABLE_TLS | ENABLE_TLS_OPENSSL */
@@ -1014,8 +1020,8 @@ engineEventLoopRun(relpEngine_t *pThis)
  * the several flavours of enhanced OS APIs.
  * rgerhards, 2008-03-17
  */
-relpRetVal
-relpEngineRun(relpEngine_t *pThis)
+relpRetVal PART_OF_API
+relpEngineRun(relpEngine_t *const pThis)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Engine);
@@ -1049,14 +1055,14 @@ PROTOTYPEcommand(S, Syslog)
  * rgerhards, 2008-03-17
  */
 relpRetVal
-relpEngineDispatchFrame(relpEngine_t *pThis, relpSess_t *pSess, relpFrame_t *pFrame)
+relpEngineDispatchFrame(relpEngine_t *const pThis, relpSess_t *pSess, relpFrame_t *pFrame)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Engine);
 	RELPOBJ_assert(pSess, Sess);
 	RELPOBJ_assert(pFrame, Frame);
 
-	pThis->dbgprint("relp engine is dispatching frame with command '%s'\n", pFrame->cmd);
+	pThis->dbgprint((char*)"relp engine is dispatching frame with command '%s'\n", pFrame->cmd);
 
 	/* currently, we hardcode the commands. Over time, they may be dynamically
 	 * loaded and, when so, should come from a linked list.
@@ -1079,7 +1085,7 @@ relpEngineDispatchFrame(relpEngine_t *pThis, relpSess_t *pSess, relpFrame_t *pFr
 	} else if(!strcmp((char*)pFrame->cmd, "serverclose")) {
 		CHKRet(relpCCServerclose(pFrame, pSess));
 	} else {
-		pThis->dbgprint("invalid or not supported relp command '%s'\n", pFrame->cmd);
+		pThis->dbgprint((char*)"invalid or not supported relp command '%s'\n", pFrame->cmd);
 		ABORT_FINALIZE(RELP_RET_INVALID_CMD);
 	}
 
@@ -1094,15 +1100,15 @@ finalize_it:
  * users should always call the client-generating functions inside the engine.
  * rgerhards, 2008-03-19
  */
-relpRetVal
-relpEngineCltConstruct(relpEngine_t *pThis, relpClt_t **ppClt)
+relpRetVal PART_OF_API
+relpEngineCltConstruct(relpEngine_t *const pThis, relpClt_t **ppClt)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Engine);
 	assert(ppClt != NULL);
 
 	CHKRet(relpCltConstruct(ppClt, pThis));
-	pThis->dbgprint("relp engine created new client %p\n", (void*) *ppClt);
+	pThis->dbgprint((char*)"relp engine created new client %p\n", (void*) *ppClt);
 
 finalize_it:
 	LEAVE_RELPFUNC;
@@ -1113,15 +1119,15 @@ finalize_it:
  * relpEngineCltConstruct(), see comment there for details.
  * rgerhards, 2008-03-19
  */
-relpRetVal
-relpEngineCltDestruct(relpEngine_t *pThis, relpClt_t **ppClt)
+relpRetVal PART_OF_API
+relpEngineCltDestruct(relpEngine_t *const pThis, relpClt_t **ppClt)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Engine);
 	assert(ppClt != NULL);
 	RELPOBJ_assert(*ppClt, Clt);
 
-	pThis->dbgprint("relp engine destructing client %p\n", (void*) *ppClt);
+	pThis->dbgprint((char*)"relp engine destructing client %p\n", (void*) *ppClt);
 	CHKRet(relpCltDestruct(ppClt));
 
 finalize_it:
@@ -1133,17 +1139,22 @@ finalize_it:
  * a configure library entry point check.
  * rgerhards, 2008-03-25
  */
-char *relpEngineGetVersion(void)
+#ifdef __GNUC__
+	/* somehow gcc-8 always gives as a wrong warning, which we so disable */
+	#pragma GCC diagnostic ignored "-Wattributes"
+#endif
+const char * PART_OF_API
+relpEngineGetVersion(void)
 {
 #	ifdef DEBUG
-		return VERSION "(debug mode)";
+		return (char*) (VERSION "(debug mode)");
 #	else
-		return VERSION;
+		return (char*) (VERSION);
 #	endif
 }
 
-void
-relpEngineSetShutdownImmdtPtr(relpEngine_t *pThis, int *ptr)
+void PART_OF_API
+relpEngineSetShutdownImmdtPtr(relpEngine_t *const pThis, int *ptr)
 {
 	if(pThis->bShutdownImmdt != ptr)
 		pThis->bShutdownImmdt = ptr;
@@ -1153,8 +1164,8 @@ relpEngineSetShutdownImmdtPtr(relpEngine_t *pThis, int *ptr)
  * it has been set to forbidden! There will be no error return state in this
  * case. -- rgerhards, 2008-03-27
  */
-relpRetVal
-relpEngineSetEnableCmd(relpEngine_t *pThis, unsigned char *pszCmd, relpCmdEnaState_t stateCmd)
+relpRetVal PART_OF_API
+relpEngineSetEnableCmd(relpEngine_t *const pThis, unsigned char *pszCmd, relpCmdEnaState_t stateCmd)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Engine);
@@ -1164,7 +1175,7 @@ relpEngineSetEnableCmd(relpEngine_t *pThis, unsigned char *pszCmd, relpCmdEnaSta
 		if(pThis->stateCmdSyslog != eRelpCmdState_Forbidden)
 			pThis->stateCmdSyslog = stateCmd;
 	} else {
-		pThis->dbgprint("tried to set unknown command '%s' to %d\n", pszCmd, stateCmd);
+		pThis->dbgprint((char*)"tried to set unknown command '%s' to %d\n", pszCmd, stateCmd);
 		ABORT_FINALIZE(RELP_RET_UNKNOWN_CMD);
 	}
 
@@ -1177,8 +1188,8 @@ finalize_it:
  * session. If disabled, the IP address will be used as the hostname.
  * rgerhards, 2008-03-31
  */
-relpRetVal
-relpEngineSetDnsLookupMode(relpEngine_t *pThis, int iMode)
+relpRetVal PART_OF_API
+relpEngineSetDnsLookupMode(relpEngine_t *const pThis, const int iMode)
 {
 	ENTER_RELPFUNC;
 	RELPOBJ_assert(pThis, Engine);
