@@ -24,7 +24,11 @@
 #include <stdlib.h>
 #include <stdarg.h>
 #include <unistd.h>
-#include <getopt.h>
+#if defined(_AIX)
+#	include  <unistd.h>
+#else
+#	include <getopt.h>
+#endif
 #include <string.h>
 #include <limits.h>
 #include <errno.h>
@@ -73,7 +77,7 @@ hdlr_enable(int sig, void (*hdlr)())
 	sigaction(sig, &sigAct, NULL);
 }
 
-void
+static void
 terminate(LIBRELP_ATTR_UNUSED const int sig)
 {
 	relpEngineSetStop(pRelpEngine);
@@ -83,7 +87,7 @@ terminate(LIBRELP_ATTR_UNUSED const int sig)
  * This method is to be "called" via ALARM after more time has expired
  * then "ever possible" - so we are sure we just need to cleanup.
  */
-void LIBRELP_ATTR_NORETURN
+static void LIBRELP_ATTR_NORETURN
 watchdog_expired(LIBRELP_ATTR_UNUSED const int sig)
 {
 	fprintf(stderr, "receive: watchdog timer expired, assuming we hang - "
@@ -93,10 +97,10 @@ watchdog_expired(LIBRELP_ATTR_UNUSED const int sig)
 }
 
 /* handler for unexpected signals.  */
-void LIBRELP_ATTR_NORETURN
+static void LIBRELP_ATTR_NORETURN
 do_signal(const int sig)
 {
-	fprintf(stderr, "send: UNEXPECTED SIGNAL %d%s- terminating\n", sig,
+	fprintf(stderr, "receive: UNEXPECTED SIGNAL %d%s- terminating\n", sig,
 		sig == SIGPIPE ? " [SIGPIPE]" : "");
 	fflush(stderr);
 	exit(100);
@@ -105,7 +109,7 @@ do_signal(const int sig)
 /* handler to unconditionally exit the code - required for test where
  * server must "suddenly" abort.
  */
-void
+static void
 do_exit(LIBRELP_ATTR_UNUSED const int sig)
 {
 	immediate_exit = 1;
@@ -149,7 +153,7 @@ static relpRetVal onSyslogRcv(unsigned char *pHostname LIBRELP_ATTR_UNUSED,
 	return RELP_RET_OK;
 }
 
-void print_usage(void)
+static void print_usage(void)
 {
 	printf("Usage: ./receive -p <PORTNUM>\n");
 }
@@ -158,14 +162,18 @@ static void
 onErr(void *pUsr, char *objinfo, char* errmesg, LIBRELP_ATTR_UNUSED relpRetVal errcode)
 {
 	struct usrdata *pThis = (struct usrdata*) pUsr;
-	if(pUsr != userdata) {
-		fprintf(stderr, "receive: pUsr NOT pointing to usrdata!\n");
+	if(pUsr != NULL) {
+		if(pUsr != userdata) {
+			fprintf(stderr, "receive: pUsr %p NOT pointing to userdata %p!\n", pUsr, (void*)userdata);
+		}
+		if(pThis->magic != USR_MAGIC) {
+			fprintf(stderr, "receive: pUsr magic incorrect in onErr, magic %8.8x "
+				"pUsr %p\n", pThis->magic, (void*) pThis);
+		}
+		fprintf(stderr, "%s: error '%s', object '%s'\n", pThis->progname, errmesg, objinfo);
+	} else {
+		fprintf(stderr, "receive: [pUsr==NULL] error '%s', object '%s'\n", errmesg, objinfo);
 	}
-	if(pThis->magic != USR_MAGIC) {
-		fprintf(stderr, "receive: pUsr magic incorrect in onErr, magic %8.8x "
-			"pUsr %p\n", pThis->magic, (void*) pThis);
-	}
-	fprintf(stderr, "%s: error '%s', object '%s'\n", pThis->progname, errmesg, objinfo);
 	if(errFile != NULL) {
 		fprintf(errFile, "receive: error '%s', object '%s'\n", errmesg, objinfo);
 	}
@@ -193,6 +201,7 @@ onAuthErr(LIBRELP_ATTR_UNUSED void *pUsr, char *authinfo,
 static void
 exit_hdlr(void)
 {
+	fprintf(stderr, "receive: EXIT HDLR\n");
 	if(userdata != NULL) {
 		free(userdata->progname);
 		free(userdata);
@@ -229,9 +238,14 @@ int main(int argc, char *argv[]) {
 	int ret = 0;
 	int append_outfile = 0;
 	int watchdog_timeout = 60; /* one seconds looks like a good default */
+	int no_exit_on_err = 0;
+	int i = 0;
 	const char *tlslib = NULL;
 	const char* outfile_name = NULL;
 
+#if defined(_AIX)
+	while((c = getopt(argc, argv, "a:c:Ae:F:l:m:No:O:P:p:TvW:x:y:z:")) != EOF) {
+#else
 	static struct option long_options[] =
 	{
 		{"ca", required_argument, 0, 'x'},
@@ -246,12 +260,12 @@ int main(int argc, char *argv[]) {
 		{"tls-lib", required_argument, 0, 'l'},
 		{"tlsconfcmd", required_argument, 0, 'c'},
 		{"watchdog-timeout", required_argument, 0, 'W'},
+		{"no-exit-on-error", no_argument, 0, 'N'},
 		{0, 0, 0, 0}
 	};
 
-
-	while((c = getopt_long(argc, argv, "a:c:Ae:F:l:m:o:O:P:p:TvW:x:y:z:",
-		long_options, &option_index)) != -1) {
+	while((c = getopt_long(argc, argv, "a:c:Ae:F:l:m:No:O:P:p:TvW:x:y:z:", long_options, &option_index)) != -1) {
+#endif
 		switch(c) {
 		case 'a':
 			authMode = optarg;
@@ -328,6 +342,9 @@ int main(int argc, char *argv[]) {
 		case 'z':
 			myPrivKeyFile = optarg;
 			break;
+		case 'N':
+			no_exit_on_err = 1;
+			break;
 		default:
 			print_usage();
 			return -1;
@@ -348,7 +365,7 @@ int main(int argc, char *argv[]) {
 
 	if(authMode != NULL) {
 		if(	(strcasecmp(authMode, "certvalid") != 0 && permittedPeer == NULL) ||
-			caCertFile == NULL || myCertFile == NULL || myPrivKeyFile == NULL) {
+			myCertFile == NULL || myPrivKeyFile == NULL) {
 			fprintf(stderr, "receive: mode '%s' parameter missing; certificates and "
 				"permittedPeer required\n", authMode);
 			goto done;
@@ -375,10 +392,14 @@ int main(int argc, char *argv[]) {
 		}
 	}
 
-	hdlr_enable(SIGTERM, terminate);
+	if (no_exit_on_err == 0) {
+		hdlr_enable(SIGPIPE, do_signal);
+	} else {
+		signal(SIGPIPE, SIG_IGN);
+	}
 	hdlr_enable(SIGUSR1, do_exit);
+	hdlr_enable(SIGTERM, terminate);
 	hdlr_enable(SIGALRM, watchdog_expired);
-	hdlr_enable(SIGPIPE, do_signal);
 
 	if(outfile_name != NULL) {
 		if((outFile = fopen(outfile_name, append_outfile ? "a" : "w")) == NULL) {
@@ -406,11 +427,13 @@ int main(int argc, char *argv[]) {
 	TRY(relpEngineSetDnsLookupMode(pRelpEngine, 0)); /* 0=disable */
 
 	TRY(relpEngineListnerConstruct(pRelpEngine, &pRelpSrv));
-	TRY(relpSrvSetLstnPort(pRelpSrv, port));
+	// Create userdata pointer as soon as possible for error callbacks
 	userdata = calloc(1, sizeof(struct usrdata));
 	userdata->magic = USR_MAGIC;
 	userdata->progname = strdup("receive");
 	relpSrvSetUsrPtr(pRelpSrv, userdata);
+
+	TRY(relpSrvSetLstnPort(pRelpSrv, port));
 	if(maxDataSize != 0) {
 		TRY(relpSrvSetMaxDataSize(pRelpSrv, maxDataSize));
 	}
@@ -455,7 +478,6 @@ int main(int argc, char *argv[]) {
 		fclose(fp);
 	}
 
-	int i = 0;
 	while(relpEngineRun(pRelpEngine) != RELP_RET_OK) {
 		fprintf(stderr, "receive: error starting relp engine, try %d\n", i);
 		++i;
