@@ -1,6 +1,6 @@
 #!/bin/bash
 . ${srcdir:=$(pwd)}/test-framework.sh
-./receive &>librelp.out.log
-cat librelp.out.log
+./receive &>${OUTFILE}
+cat ${OUTFILE}
 check_output "Port is missing"
 terminate
